// implementation of static repository

use super::domain::*;
use super::state::AdminRegistry;
use log::warn;
use serde_json::{json, Value};

struct MyStaticRepository {
    content: Vec<Value>,
}

impl MyStaticRepository {
    pub fn new() -> Self {
        MyStaticRepository {
            content: vec![
                json!({"id": 1, "name": "Strange", "age": 150, "level": "master" }),
                json!({"id": 2, "name": "Adam", "age": 12, "powers": 8}),
                json!({"id": 3, "name": "Tony", "age": 42, "powers": 0}),
                json!({"id": 4, "name": "Rex", "age": 72}),
                json!({"id": 5, "name": "Justin", "age": 46}),
                json!({"id": 6, "name": "Reacher", "age": 39, "level": "adept", "powers": 0}),
                json!({"id": 7, "name": "Arnold", "age": 7}),
                json!({"id": 8, "name": "Eight", "age": 8}),
                json!({"id": 9, "name": "Nine", "age": 9}),
                json!({"id": 10, "name": "Ten", "age": 10}),
                json!({"id": 11, "name": "Eleven", "age": 11}),
            ],
        }
    }
}

impl AdminRepository for MyStaticRepository {
    fn get(&self, model: &AdminModel, id: LookupKey) -> Option<RepositoryItem> {
        if let LookupKey::Integer(id) = id {
            let item = self.content.get(id - 1).cloned().unwrap();
            Some(RepositoryItem {
                detail_url: Some(format!(
                    "{}/detail/{}",
                    model.admin_url,
                    item.get("id").unwrap()
                )),
                fields: item,
            })
        } else {
            warn!("Got non-integer lookup key: {}", id);
            None
        }
    }

    fn list(&self, model: &AdminModel) -> RepositoryList {
        // Admin needs to inject info in these.
        RepositoryList::List {
            values: self
                .content
                .clone()
                .into_iter()
                .map(|item| RepositoryItem {
                    detail_url: Some(format!(
                        "{}/detail/{}",
                        model.admin_url,
                        item.get("id").unwrap()
                    )),
                    fields: item,
                })
                .collect(),
        }
    }

    fn info(&self, model: &AdminModel) -> RepositoryInfo {
        RepoInfo {
            name: "My Static Repository",
            lookup_key: "id",
            display_list: &["id", "name", "level", "age"],
        }
        .into()
    }

    fn create(&self, model: &AdminModel, data: Value) -> Option<RepositoryItem> {
        println!("I would now create: {}", data);
        None
    }

    fn update(&self, model: &AdminModel, id: LookupKey, data: Value) -> Option<RepositoryItem> {
        println!("I would now update: {}, {}", id, data);
        None
    }

    fn delete(&self, model: &AdminModel, id: LookupKey) -> Option<Value> {
        println!("Would delete: {}", id);
        None
    }
}

pub fn register_example(registry: &mut AdminRegistry) {
    let app_key = registry.register_app("Example App");
    let repo = MyStaticRepository::new();
    let model_config = AdminModelConfig {
        app_key: app_key,
        name: "ExampleModel".to_owned(),
    };
    let model_result = registry.register_model(model_config, repo);
    match model_result {
        Err(err) => panic!("{}", err),
        _ => (),
    }
}
